function [s, data, filts, mtrx] = check_signals(t0, data, chans, filts, mtrx)
  % check the FE signals in the DARM chain at a particular GPS time
  %   [s, data, mtrx] = check_signals(t0, data, chans, filts, mtrx)
  %
  % t0 = gps time of interest.  This need not be an integer.
  % {data, chans} = output of fetch_data, or the output of resample_data
  %    if chans is empty (e.g., chans = []).  This will be loaded
  %    automatically if not given as an argument, but it is usually
  %    better to call fetch_data externally to avoid fetching the data
  %    multiple times.
  % {filts, mtrx} = output of fetch_state (will be loaded if not given)
  %    if mtrx is the output of measure_gains, that will function will
  %    not be called again here (making this a bit faster).
  %
  % The output values are:
  % s = struct containing all computed signals
  % data = output of resample_data
  % mtrx = output of measure_gains

  % IFO name
  ifo_name = 'H1';
  is_debug = false;
  n_delay2k = 15; % 28
  n_delay1k = 32; % 58
  n_delay_mon2 = 41; % 37
  n_delay_mon3 = 85;
  
  % fetch data
  if nargin < 3
    disp('Fetching event data...')
    dt = 10;
    t_off = 8;
    [data, chans] = fetch_data(ifo_name, round(t0) - t_off, dt);
  end
  
  % fetch filter states
  if nargin < 5
    disp('Fetching system state...')
    [filts, mtrx] = fetch_state(ifo_name, t0);
  end

  % resample all data to 16384, and apply bucket filter
  if ~isempty(chans)
    disp('Resampling data...')
    data = resample_data(data, chans, t0);
  end
  
  % measure gain factors
  if ~isfield(mtrx, 'g_SUM_DARM')
    disp('Measuring gains...')
    mtrx = measure_gains(data, filts, mtrx);
  end
  
  % timing
  fs = data.fs;
  t = data.t;
  
  s.sum = filts.OMC_DCPD_SUM.filter(data.DCPD_A_OUT + data.DCPD_B_OUT);
  s.null = data.DCPD_A_OUT - data.DCPD_B_OUT;  % should have NULL filter
  is_same(data.DCPD_SUM_OUT, s.sum, 'SUM', t, fs, 0, is_debug)
  %is_same(data.DCPD_NULL_OUT, s.null, 'NULL', t, fs, 0, is_debug)
    
  % compare SUM with DARM IN
  flt = filts.OMC_READOUT_ERR * filts.LSC_OMC_DC;
  %s.darm_in = flt.filter(data.DCPD_SUM_OUT) * (mtrx.g_SUM_DARM * mtrx.LSC_ARM_INPUT_MTRX_1_1);
  s.darm_in = flt.filter(s.sum) * (mtrx.g_SUM_DARM * mtrx.LSC_ARM_INPUT_MTRX_1_1);
  is_same(data.DARM_IN1, s.darm_in, 'DARM IN', t, fs, 0, is_debug)
  
  % filter DARM_IN, and compare with DARM_OUT  
  %s.darm_out = filts.LSC_DARM.filter(data.DARM_IN1);
  s.darm_out = filts.LSC_DARM.filter(s.darm_in);
  is_same(data.DARM_OUT, s.darm_out, 'DARM OUT', t, fs, 0, is_debug)
  
  % input to SUS ETMY 
  %s.etmy_in1 = data.DARM_OUT * mtrx.LSC_ARM_OUTPUT_MTRX_2_1;
  s.etmy_in1 = s.darm_out * mtrx.LSC_ARM_OUTPUT_MTRX_2_1;
  is_same(data.ETMY_L3_ISCINF_L_IN1, s.etmy_in1, 'ETMY IN1', t, fs, 1, is_debug)
  
  % input to L3 LOCK  (ETMY_L3_ISCINF_L_IN1 is last signal at 16384)
  %s.etmy_in2 = filts.SUS_ETMY_L3_ISCINF_L.filter(data.ETMY_L3_ISCINF_L_IN1);
  s.etmy_in2 = filts.SUS_ETMY_L3_ISCINF_L.filter(s.etmy_in1);
  is_same(data.ETMY_L3_LOCK_L_IN1, s.etmy_in2, 'ETMY IN2', t, fs, n_delay2k, is_debug)
  
  % output of L3 LOCK  
  s.etmy_L3 = filts.SUS_ETMY_L3_LOCK_L.filter(s.etmy_in2);
  is_same(data.ETMY_L3_LOCK_L_OUT, s.etmy_L3, 'ETMY L3', t, fs, n_delay2k, is_debug)
  
  % output of L2 LOCK  
  s.etmy_L2 = filts.SUS_ETMY_L2_LOCK_L.filter(s.etmy_L3);
  is_same(data.ETMY_L2_LOCK_L_OUT, s.etmy_L2, 'ETMY L2', t, fs, n_delay2k, is_debug)
  
  % output of L1 LOCK  
  s.etmy_L1 = filts.SUS_ETMY_L1_LOCK_L.filter(s.etmy_L2);
  is_same(data.ETMY_L1_LOCK_L_OUT, s.etmy_L1, 'ETMY L1', t, fs, n_delay1k, is_debug)
  
  % filters for noise mons
  flt_nm2 = GWFilt.zpg([0, 0, 23], [5, 7, 300], 11.5, 100);
  flt_nm2 = flt_nm2.set_fs(fs);
  flt_nm3 = GWFilt.zpg([50, 50], [2.2, 2.2, 42], 0.1, 1);
  flt_nm3 = flt_nm3.set_fs(fs) * GWFilt.butter(80, [], fs, 8); % add AA;
  
  % filter DARM to get L3 SUS outputs
  seg_names = {'LL', 'LR', 'UL', 'UR'};
  for n = 1:4
    sn = seg_names{n};
    eul = ['SUS_ETMY_L3_EUL2ESD_' num2str(n) '_1'];
    flt = mtrx.(eul) * mtrx.g_ETMY_L3 * ...
      filts.SUS_ETMY_L3_DRIVEALIGN_L2L * ...
      filts.(['SUS_ETMY_L3_ESDOUTF_' sn]);
    s.outL3(:, n) = flt.filter(s.etmy_L3);
    s.monL3(:, n) = flt_nm3.filter(s.outL3(:, n));
    %s.monL3(:, n) = flt_nm3.filter(data.(['ETMY_L3_MASTER_OUT_' sn]));
    is_same(data.(['ETMY_L3_MASTER_OUT_' sn]), s.outL3(:, n), ...
      ['OUT L3 ' sn], t, fs, n_delay2k, is_debug)
    is_same(data.(['ETMY_L3_LVESDAMON_' sn]), s.monL3(:, n), ...
      ['MON L3 ' sn], t, fs, n_delay_mon3, is_debug)
  end
  
  % filter DARM to get L2 SUS outputs
  seg_names = {'LL', 'LR', 'UL', 'UR'};
  for n = 1:4
    sn = seg_names{n};
    eul = ['SUS_ETMY_L2_EUL2OSEM_' num2str(n) '_1'];
    flt = mtrx.(eul) * filts.SUS_ETMY_L2_DRIVEALIGN_L2L * ...
      filts.(['SUS_ETMY_L2_COILOUTF_' sn]);
    s.outL2(:, n) = flt.filter(s.etmy_L2);
    s.monL2(:, n) = flt_nm2.filter(s.outL2(:, n));
    %s.monL2(:, n) = flt_nm2.filter(data.(['ETMY_L2_MASTER_OUT_' sn]));
    is_same(data.(['ETMY_L2_MASTER_OUT_' sn]), s.outL2(:, n), ...
      ['OUT L2 ' sn], t, fs, n_delay2k, is_debug)
    is_same(data.(['ETMY_L2_NOISEMON_' sn]), s.monL2(:, n), ...
      ['MON L2 ' sn], t, fs, n_delay_mon2, is_debug)
  end
  
  % filter DARM to get L1 SUS outputs
  seg_names = {'LL', 'LR', 'UL', 'UR'};
  for n = 1:4
    sn = seg_names{n};
    eul = ['SUS_ETMY_L1_EUL2OSEM_' num2str(n) '_1'];
    flt = mtrx.(eul) * filts.SUS_ETMY_L1_DRIVEALIGN_L2L * ...
      filts.(['SUS_ETMY_L1_COILOUTF_' sn]);
    s.outL1(:, n) = flt.filter(s.etmy_L1);
    is_same(data.(['ETMY_L1_MASTER_OUT_' sn]), s.outL1(:, n), ...
      ['OUT L1 ' sn], t, fs, n_delay1k, is_debug)
  end
  
  %%%%%%%%%%%%%%%%%%%%%%% ETMX (stub)
  % input to SUS ETMX
  s.etmx_in1 = s.darm_out * mtrx.LSC_ARM_OUTPUT_MTRX_1_1;
  is_same(data.ETMX_L3_ISCINF_L_IN1, s.etmx_in1, 'ETMX IN1', t, fs, 1, is_debug)
  
  % input to L3 LOCK  (ETMX_L3_ISCINF_L_IN1 is last signal at 16384)
  %s.etmx_in2 = filts.SUS_ETMX_L3_ISCINF_L.filter(data.ETMX_L3_ISCINF_L_IN1);
  s.etmx_in2 = filts.SUS_ETMX_L3_ISCINF_L.filter(s.etmx_in1);
  is_same(data.ETMX_L3_LOCK_L_IN1, s.etmx_in2, 'ETMX IN2', t, fs, n_delay2k, is_debug)
  
  % output of L3 LOCK is zero
  %s.etmx_L3 = filts.SUS_ETMX_L3_LOCK_L.filter(s.etmx_in2);
  %is_same(data.ETMX_L3_LOCK_L_OUT, s.etmx_L3, 'ETMX L3', t, fs, n_delay2k, is_debug)
  
  %%%%%%%%%%%%%%%%%%%%%%% PCAL
  plot(data.t, [data.PCALY_RX_PD_OUT, data.PCALX_RX_PD_OUT] * 1e13);
  ylim([-2, 2])
  xlim([-0.2, 0.2])
  grid on
  title('PCAL RX')
  legend('PCAL Y', 'PCAL X')
  xlabel('time [s]')
  orient rotated
  print('-dpdf', ['plots/PCAL.pdf'])
end

function is_same(a, b, name, t, fs, n_delay, is_debug)
  [hab, ff] = tfestimate(a, b, hann(8 * fs), [], 8 * fs, fs);
  
  % trim a few seconds
  ntrim = 6 * fs;
  a = a(ntrim:end);
  b = b((ntrim:end) - n_delay);
  t = t(ntrim:end);
  rms = sqrt(abs(GWFilt.butter((a - b).^2, 100, [], fs)));  
  
  if is_debug
    nf = ff > 50 & ff < 200;
    %nf = ff > 20 & ff < 50;
    %nf = ff > 4 & ff < 6;
    t_delay = unwrap(angle(hab(nf))) ./ (2 * pi * ff(nf));
    t_delay = mean(t_delay);
    fprintf(1, '%s delay is %.1f cycles (got %d)\n', name, t_delay * fs, n_delay);
    
    subplot(2, 1, 1)
    plot(t, [a, b, rms])
    title(name)
    grid on
    legend('FE data', 'simulation', 'residual')
    xlabel('time [s]')
    
    subplot(2, 1, 2)
    loglog(ff, abs(hab))
    ylim([0.1, 10])
    xlim([1, 1000])
    grid on
    pause
  else
    clf
    plot(t, [a, b, rms])
    title(name)
    grid on
    legend('FE data', 'simulation', 'residual')
    xlabel('time [s]')
    xlim([-0.2, 0.2])
    orient rotated
    print('-dpdf', ['plots/' strrep(name, ' ', '_') '.pdf'])
  end
end
